#include <fstream>
#include <iostream>

#include "main.hpp"

int main(int argc, char **argv) {

  /* declarations */
  int result = 0;

  /* Parsing CLI Arguments */
  po::options_description desc("Available options");
  desc.add_options()("help,h", "show help")("verbose,v", "more verbose output")(
      "vert", po::value<string>()->required(), "vertex shader file")(
      "frag", po::value<string>()->required(), "fragment shader file");

  /* unused for now */
  po::positional_options_description pdesc;
  // NOTE: if you want to add a positional argument,
  // you need to create and option for it as it seems.
  pdesc.add("vert", 1);
  pdesc.add("frag", 1);

  po::variables_map vm;
  try {

    po::store(po::command_line_parser(argc, argv)
                  .options(desc)     // load options
                  .positional(pdesc) // load positionals
                  .run(),
              vm);
    po::notify(vm);
    if (vm.count("help") || argc < 3) {
      help(argv[0], desc);
      return 1;
    }
  } catch (po::error &e) {
    BOOST_LOG_TRIVIAL(fatal) << e.what() << endl << endl;
    help(argv[0], desc);
    return EXIT_USAGE;
  }

  // set boost log level to trace on verbose
  // TODO: -v should decrease the default log level by 1 but be repeatable.
  // (default should be info)
  if (vm.count("verbose")) {
    boost::log::core::get()->set_filter(boost::log::trivial::severity >=
                                        boost::log::trivial::trace);
    BOOST_LOG_TRIVIAL(trace) << "verbose flag found, setting log level to trace";
  }

  /* init graphics stuff */
  result = initGl();
  if (result != 0) { return EXIT_GL;}

  /* setup the shader program */
  BOOST_LOG_TRIVIAL(trace) << "Reading shader files";
  ifstream vertexFileStream(vm["vert"].as<string>());
  ifstream fragFileStream(vm["frag"].as<string>());
  std::stringstream vertexSource;
  std::stringstream fragSource;
  vertexSource << vertexFileStream.rdbuf();
  fragSource << fragFileStream.rdbuf();

  if (vertexFileStream.fail()) {
    BOOST_LOG_TRIVIAL(fatal)
        << "Error while reading the vertex shader file.";
    return EXIT_IO;
  }
  if (fragFileStream.fail()) {
    BOOST_LOG_TRIVIAL(fatal)
        << "Error while reading the frag shader file.";
    return EXIT_IO;
  }
  vertexFileStream.close();
  fragFileStream.close();

  BOOST_LOG_TRIVIAL(trace) << "Creating ShaderProgram";

  // TODO: determinde ShaderProgram::Base with a CLI arg
  ShaderProgram shaderProgram(ShaderProgram::Base::Empty, vertexSource.str(),
                              fragSource.str());

  /* Run our main program */
  BOOST_LOG_TRIVIAL(trace) << "Starting mainWindow";
  result = mainWindow(&shaderProgram);
  /* if (result != 0) { return EXIT_GL;} */

  return result;
}

void help(char *prog, po::options_description desc) {
  cout << "Usage:" << endl
       << prog
       << " [-vh] [--vert] shaders/vertex.glsl [--frag] shaders/fragment.glsl"
       << endl
       << endl
       << "return values:" << endl
       << "0\tSuccess"
       << "1\tGeneric Failure"
       << "2\tBad arguments"
       << "3\tI/O Error" << endl
       << "3\tOpenGL Error" << endl
       << endl
       << desc << endl;
  // NOTE: constants for the error codes are defined in main.hpp
}
